package com.ruoyi.web.hksdk;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ruoyi.web.cache.CacheManager;
import com.ruoyi.web.component.CaputreSetting;
import com.sun.jna.ptr.IntByReference;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@Component
@Slf4j
public class HkAdapter {
    @Resource
    private CaputreSetting caputreSetting;
    @Resource
    private HkUtils hkUtils;
    @Resource
    private HCNetSDK hcNetSDK;


    /**
     * 查询人员信息
     */
    public Map<String,String> eventUserInfo(String equipmentNo) {
        log.info("---------------------查询人员信息---开始----------------------------");
        int userId= CacheManager.getUserId();
        //int userId = hkUtils.Login_V40(caputreSetting.getHost(), (short) caputreSetting.getPort(), caputreSetting.getUsername(), caputreSetting.getPwd(), hcNetSDK);
        //数组

        HCNetSDK.BYTE_ARRAY ptrByteArray = new HCNetSDK.BYTE_ARRAY(1024);
        String url = "POST /ISAPI/AccessControl/UserInfo/Search?format=json";
        //字符串拷贝到数组中
        System.arraycopy(url.getBytes(), 0, ptrByteArray.byValue, 0, url.length());
        ptrByteArray.write();

        int lHandler = hcNetSDK.NET_DVR_StartRemoteConfig(userId, HCNetSDK.NET_DVR_JSON_CONFIG, ptrByteArray.getPointer(), url.length(), null, null);
        if (lHandler < 0) {
            log.info("SearchUserInfo失败,错误码为" + hcNetSDK.NET_DVR_GetLastError());
        } else {
            JSONObject jsonObject = new JSONObject();
            JSONObject jsonSearchCond = new JSONObject();

            // 查询指定的工号人员信息
            JSONArray jsonArray = new JSONArray();
            if (!ObjectUtils.isEmpty(equipmentNo)) {
                JSONObject json = new JSONObject();
                json.put("employeeNo", equipmentNo);
                jsonArray.add(json);
            }

            jsonSearchCond.put("EmployeeNoList", jsonArray);
            jsonSearchCond.put("searchID", DateUtil.format(new Date(), DatePattern.PURE_DATETIME_MS_FORMAT));
            jsonSearchCond.put("searchResultPosition", 0);
            jsonSearchCond.put("maxResults", 1);
            jsonObject.put("UserInfoSearchCond", jsonSearchCond);

            String toString = jsonObject.toString();
            log.info("查询的json报文:" + toString);

            //把string传递到Byte数组中，后续用.getPointer()方法传入指针地址中。
            HCNetSDK.BYTE_ARRAY byteArray = new HCNetSDK.BYTE_ARRAY(toString.length());
            System.arraycopy(toString.getBytes(), 0, byteArray.byValue, 0, toString.length());
            byteArray.write();

            //定义接收结果的结构体
            HCNetSDK.BYTE_ARRAY ptrOutBuff = new HCNetSDK.BYTE_ARRAY(10 * 1024);
            IntByReference pInt = new IntByReference(0);

            while (true) {
                int dwState = hcNetSDK.NET_DVR_SendWithRecvRemoteConfig(lHandler, byteArray.getPointer(), toString.length(), ptrOutBuff.getPointer(), 20 * 1024, pInt);
                if (dwState == -1) {
                    break;
                } else if (dwState == HCNetSDK.NET_SDK_CONFIG_STATUS_NEED_WAIT) {
                    try {
                        Thread.sleep(10);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                } else if (dwState == HCNetSDK.NET_SDK_CONFIG_STATUS_FAILED) {
                    log.info("查询人员失败");
                    break;
                } else if (dwState == HCNetSDK.NET_SDK_CONFIG_STATUS_EXCEPTION) {
                    log.info("查询人员异常");
                    break;
                } else if (dwState == HCNetSDK.NET_SDK_CONFIG_STATUS_SUCCESS) {
                    ptrOutBuff.read();
                    String trim = new String(ptrOutBuff.byValue).trim();
                    // 得到的人员信息
                    try {
                        ObjectMapper objectMapper = new ObjectMapper();
                        JsonNode jsonNode = objectMapper.readTree(trim);
                        JsonNode userInfoSearch = jsonNode.get("UserInfoSearch");
                        JsonNode userInfo = userInfoSearch.get("UserInfo");
                        JsonNode userInfoOne = userInfo.get(0);
                        JsonNode name = userInfoOne.get("name");
                        JsonNode roomNumber = userInfoOne.get("roomNumber");
                        JsonNode floorNumber = userInfoOne.get("floorNumber");
                        Map<String,String> map = new HashMap<>();
                        map.put("name",name.asText());
                        map.put("roomNumber",roomNumber.asText());
                        map.put("floorNumber",floorNumber.asText());
                        log.info("---------------------查询人员信息---结束1----------------------------");
                        return map;
                    } catch (JsonProcessingException e) {
                        log.info("查询人员信息报错:{}",e.getMessage());
                        log.info("---------------------查询人员信息---结束2----------------------------");
                        return null;
                    }
                } else if (dwState == HCNetSDK.NET_SDK_CONFIG_STATUS_FINISH) {
                    log.info("获取人员完成");
                    break;
                }
            }

            if (!hcNetSDK.NET_DVR_StopRemoteConfig(lHandler)) {
                log.error("NET_DVR_StopRemoteConfig接口调用失败，错误码：" + hcNetSDK.NET_DVR_GetLastError());
            } else {
                log.info("NET_DVR_StopRemoteConfig接口成功");
            }
        }
        log.info("---------------------查询人员信息---结束3----------------------------");
        return null;
    }

}
