package com.ruoyi.web.service;

import cn.hutool.extra.spring.SpringUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.ruoyi.web.hksdk.HCNetSDK;
import com.ruoyi.web.hksdk.HkAdapter;
import com.sun.jna.Pointer;
import lombok.extern.slf4j.Slf4j;

import java.nio.ByteBuffer;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZoneOffset;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Map;

@Slf4j
public class InterfaceService implements Runnable {

    private HCNetSDK.NET_DVR_ALARMER pAlarmer;
    private Pointer pAlarmInfo;

    public InterfaceService(HCNetSDK.NET_DVR_ALARMER pAlarmer, Pointer pAlarmInfo) {
        this.pAlarmer = pAlarmer;
        this.pAlarmInfo = pAlarmInfo;
    }

    @Override
    public void run() {
        String[] sIP = new String[2];
        sIP = new String(pAlarmer.sDeviceIP).split("\0", 2);
        //报警设备ip
        String ip = sIP[0];
        String eventTime = null;
        String employeeId = null;
        String cardNo = null;
        Byte cardType = null;

        HCNetSDK.NET_DVR_ACS_ALARM_INFO strACSInfo = new HCNetSDK.NET_DVR_ACS_ALARM_INFO();
        strACSInfo.write();
        Pointer pACSInfo = strACSInfo.getPointer();
        pACSInfo.write(0, pAlarmInfo.getByteArray(0, strACSInfo.size()), 0, strACSInfo.size());
        strACSInfo.read();
        //主类型 5 事件上报
        if (strACSInfo.dwMajor != 5) {
            return;
        }

        byte timeType = strACSInfo.byTimeType;
        HCNetSDK.NET_DVR_TIME struTime = strACSInfo.struTime;
        LocalDateTime uplaodTime = LocalDateTime.of(struTime.dwYear, struTime.dwMonth, struTime.dwDay, struTime.dwHour, struTime.dwMinute, struTime.dwSecond);
        DateTimeFormatter fmt = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        if (timeType == 0) {
            eventTime = uplaodTime.format(fmt);
        } else if (timeType == 1) {
            ZonedDateTime zonedDateTime = uplaodTime.atZone(ZoneId.from(ZoneOffset.UTC)).withZoneSameInstant(ZoneId.systemDefault());
            eventTime = zonedDateTime.format(fmt);
        } else {
            log.info("上报时间不正确");
        }

        cardNo = new String(strACSInfo.struAcsEventInfo.byCardNo).trim();
        // cardType = strACSInfo.struAcsEventInfo.byCardType;

        //以人为中心设备
        if (strACSInfo.byAcsEventInfoExtend == 1) {
            HCNetSDK.NET_DVR_ACS_EVENT_INFO_EXTEND test = new HCNetSDK.NET_DVR_ACS_EVENT_INFO_EXTEND();
            test.write();
            Pointer pAcsEventInfoExtend = test.getPointer();
            pAcsEventInfoExtend.write(0, strACSInfo.pAcsEventInfoExtend.getByteArray(0, test.size()), 0, test.size());
            test.read();
            employeeId = new String(test.byEmployeeNo).trim();
        }
        HkAdapter hkAdapter = SpringUtil.getBean(HkAdapter.class);
        Map<String, String> map = hkAdapter.eventUserInfo(employeeId);
        String name = map.get("name");
        log.info("员工姓名name="+name);
        String roomNumber = map.get("roomNumber");
        log.info("房间号="+roomNumber);

        if (strACSInfo.dwPicDataLen > 0) {
            long offset = 0;
            ByteBuffer buffers = strACSInfo.pPicData.getByteBuffer(offset, strACSInfo.dwPicDataLen);
            byte[] bytes = new byte[strACSInfo.dwPicDataLen];
            buffers.rewind();
            buffers.get(bytes);
            log.info("有图片");
        }

        //密码认证通过
        if (strACSInfo.dwMinor == 181) {
            log.info("密码认证通过============dwMinor============" + strACSInfo.dwMinor);
            req(cardNo, name, eventTime);
        }
        //密码认证失败
        else if (strACSInfo.dwMinor == 151) {
            log.info("密码认证失败============dwMinor============" + strACSInfo.dwMinor);
        }
        //刷脸成功
        else if (strACSInfo.dwMinor == 75) {
            log.info("刷脸成功============dwMinor============" + strACSInfo.dwMinor);
            req(cardNo, name, eventTime);
        }
        //刷脸失败
        else if (strACSInfo.dwMinor == 76) {
            log.info("刷脸失败============dwMinor============" + strACSInfo.dwMinor);
        }
        //门锁打开
        else if (strACSInfo.dwMinor == 21) {
            log.info("门锁打开============dwMinor============" + strACSInfo.dwMinor);
        }
        //门锁关闭
        else if (strACSInfo.dwMinor == 22) {
            log.info("门锁关闭============dwMinor============" + strACSInfo.dwMinor);
        }
        // 刷卡成功
        else if (strACSInfo.dwMinor == 1) {
            log.info("刷卡成功============dwMinor============" + strACSInfo.dwMinor);
        } else {
            log.info("(不是刷脸也不是密码打卡，打卡无效)dwMinor============" + strACSInfo.dwMinor);
            req(cardNo, name, eventTime);
        }
    }

    public void req(String cardNo, String name, String eventTime) {
        ReqBO reqBO = new ReqBO();
        reqBO.setCarNo(cardNo).setReqTime(eventTime).setName(name);
        String resp = HttpUtil.post("http://127.0.0.1:9090/api/req", JSONUtil.toJsonStr(reqBO));
       log.info("resp:{}",resp);

    }


}
